@@ -4,7 +4,7 @@
 // Author:      Paul Lanier
 // Modified by:
 // Created:     05/25/06
-// RCS-ID:      $Id: pseudodc.cpp 42187 2006-10-21 01:22:11Z RD $
+// RCS-ID:      $Id: pseudodc.cpp 51090 2008-01-08 04:36:23Z RD $
 // Copyright:   (c) wxWidgets team
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
@@ -236,7 +236,7 @@ pdcDrawSplineOp::~pdcDrawSplineOp()
 // ----------------------------------------------------------------------------
 void pdcObject::DrawToDC(wxDC *dc)
 {
-    pdcOpList::Node *node = m_oplist.GetFirst(); 
+    pdcOpList::compatibility_iterator node = m_oplist.GetFirst(); 
     while(node)
     {
         node->GetData()->DrawToDC(dc, m_greyedout);
@@ -249,17 +249,17 @@ void pdcObject::DrawToDC(wxDC *dc)
 // ----------------------------------------------------------------------------
 void pdcObject::Translate(wxCoord dx, wxCoord dy)
 {
-    pdcOpList::Node *node = m_oplist.GetFirst(); 
+    pdcOpList::compatibility_iterator node = m_oplist.GetFirst(); 
     while(node)
     {
         node->GetData()->Translate(dx,dy);
         node = node->GetNext();
     }
-	if (m_bounded) 
-	{
-		m_bounds.x += dx;
-		m_bounds.y += dy;
-	}
+    if (m_bounded) 
+    {
+        m_bounds.x += dx;
+        m_bounds.y += dy;
+    }
 }
 
 // ----------------------------------------------------------------------------
@@ -268,19 +268,18 @@ void pdcObject::Translate(wxCoord dx, wxCoord dy)
 // ----------------------------------------------------------------------------
 void pdcObject::SetGreyedOut(bool greyout) 
 {
-	m_greyedout=greyout;
-	if (greyout)
-	{
-		pdcOpList::Node *node = m_oplist.GetFirst(); 
+    m_greyedout=greyout;
+    if (greyout)
+    {
+        pdcOpList::compatibility_iterator node = m_oplist.GetFirst(); 
         pdcOp *obj;
-		while(node)
-		{
-
-			obj = node->GetData();
+        while(node)
+        {
+            obj = node->GetData();
             obj->CacheGrey();
-			node = node->GetNext();
-		}
-	}
+            node = node->GetNext();
+        }
+    }
 }
 
 // ============================================================================
@@ -303,8 +302,10 @@ wxPseudoDC::~wxPseudoDC()
 void wxPseudoDC::RemoveAll(void)
 {
     m_objectlist.Clear();
+    m_objectIndex.clear();
     m_currId = -1;
-	m_lastObjNode = NULL;
+	m_lastObject = NULL;
+    
 }
 
 // ----------------------------------------------------------------------------
@@ -312,7 +313,7 @@ void wxPseudoDC::RemoveAll(void)
 // ----------------------------------------------------------------------------
 int wxPseudoDC::GetLen(void)
 {
-    pdcObjectList::Node *pt = m_objectlist.GetFirst();
+    pdcObjectList::compatibility_iterator pt = m_objectlist.GetFirst();
     int len=0;
     while (pt) 
     {
@@ -323,37 +324,30 @@ int wxPseudoDC::GetLen(void)
 }
 
 // ----------------------------------------------------------------------------
-// FindObjNode - find and return an object node by id.  If node doesn't exist
+// FindObject - find and return an object node by id.  If node doesn't exist
 //               and create is true then create one and return it.  Otherwise
 //               return NULL.
 // ----------------------------------------------------------------------------
-pdcObjectList::Node *wxPseudoDC::FindObjNode(int id, bool create)
+pdcObject *wxPseudoDC::FindObject(int id, bool create)
 {
     // see if last operation was for same id
-    if (m_lastObjNode && m_lastObjNode->GetData()->GetId() == id)
-        return m_lastObjNode;
-    // if not then search for it
-    pdcObjectList::Node *pt = m_objectlist.GetFirst();
-    while (pt) 
-    {
-        if (pt->GetData()->GetId() == id)
-		{
-        
-            // cache this node for future operations
-            m_lastObjNode = pt;
-            return pt;
+    //~ if (m_lastObject && m_lastObject->GetId() == id)
+        //~ return m_lastObject;
+    // if not then search for it    
+    pdcObjectHash::iterator lookup = m_objectIndex.find(id);
+    if (lookup == m_objectIndex.end()) {//not found
+        if (create) {
+            m_lastObject = new pdcObject(id);
+            m_objectlist.Append(m_lastObject);
+			pdcObjectHash::value_type insert(id, m_lastObject);
+            m_objectIndex.insert(insert);
+            return m_lastObject;
+        } else {
+            return NULL;
         }
-        pt = pt->GetNext();
+    } else { //found
+        return lookup->second;
     }
-    // if create then create and return a new node
-    if (create) 
-    {
-        // cache this node for future operations
-        m_lastObjNode = m_objectlist.Append(new pdcObject(id));
-        return m_lastObjNode;
-    }
-    // otherwise just return NULL
-    return NULL;
 }
 
 // ----------------------------------------------------------------------------
@@ -361,8 +355,8 @@ pdcObjectList::Node *wxPseudoDC::FindObjNode(int id, bool create)
 // ----------------------------------------------------------------------------
 void wxPseudoDC::AddToList(pdcOp *newOp)
 {
-    pdcObjectList::Node *pt = FindObjNode(m_currId, true);
-    pt->GetData()->AddOp(newOp);
+    pdcObject *obj = FindObject(m_currId, true);
+    obj->AddOp(newOp);
 }
 
 // ----------------------------------------------------------------------------
@@ -370,8 +364,8 @@ void wxPseudoDC::AddToList(pdcOp *newOp)
 // ----------------------------------------------------------------------------
 void wxPseudoDC::ClearId(int id)
 {
-    pdcObjectList::Node *pt = FindObjNode(id);
-    if (pt) pt->GetData()->Clear();
+    pdcObject *obj = FindObject(id);
+    if (obj) obj->Clear();
 }
 
 // ----------------------------------------------------------------------------
@@ -379,13 +373,14 @@ void wxPseudoDC::ClearId(int id)
 // ----------------------------------------------------------------------------
 void wxPseudoDC::RemoveId(int id)
 {
-    pdcObjectList::Node *pt = FindObjNode(id);
-    if (pt) 
+    pdcObject *obj = FindObject(id);
+    if (obj) 
     {
-        if (m_lastObjNode == pt)
-            m_lastObjNode = NULL;
-        m_objectlist.DeleteNode(pt);
+        if (m_lastObject == obj)
+            m_lastObject = obj;
+        m_objectlist.DeleteObject(obj);
     }
+    m_objectIndex.erase(id);
 }
 
 // ----------------------------------------------------------------------------
@@ -393,8 +388,8 @@ void wxPseudoDC::RemoveId(int id)
 // ----------------------------------------------------------------------------
 void wxPseudoDC::SetIdBounds(int id, wxRect& rect)
 {
-    pdcObjectList::Node *pt = FindObjNode(id, true);
-    pt->GetData()->SetBounds(rect);
+    pdcObject *obj = FindObject(id, true);
+    obj->SetBounds(rect);
 }
 
 // ----------------------------------------------------------------------------
@@ -402,9 +397,9 @@ void wxPseudoDC::SetIdBounds(int id, wxRect& rect)
 // ----------------------------------------------------------------------------
 void wxPseudoDC::GetIdBounds(int id, wxRect& rect)
 {
-    pdcObjectList::Node *pt = FindObjNode(id);
-	if (pt && pt->GetData()->IsBounded())
-		rect = pt->GetData()->GetBounds();
+    pdcObject *obj = FindObject(id);
+	if (obj && obj->IsBounded())
+		rect = obj->GetBounds();
 	else
 		rect.x = rect.y = rect.width = rect.height = 0;
 }
@@ -414,8 +409,8 @@ void wxPseudoDC::GetIdBounds(int id, wxRect& rect)
 // ----------------------------------------------------------------------------
 void wxPseudoDC::TranslateId(int id, wxCoord dx, wxCoord dy)
 {
-    pdcObjectList::Node *pt = FindObjNode(id);
-    if (pt) pt->GetData()->Translate(dx,dy);
+    pdcObject *obj = FindObject(id);
+    if (obj) obj->Translate(dx,dy);
 }
 
 // ----------------------------------------------------------------------------
@@ -423,8 +418,8 @@ void wxPseudoDC::TranslateId(int id, wxCoord dx, wxCoord dy)
 // ----------------------------------------------------------------------------
 void wxPseudoDC::DrawIdToDC(int id, wxDC *dc)
 {
-    pdcObjectList::Node *pt = FindObjNode(id);
-    if (pt) pt->GetData()->DrawToDC(dc);
+    pdcObject *obj = FindObject(id);
+    if (obj) obj->DrawToDC(dc);
 }
 
 // ----------------------------------------------------------------------------
@@ -432,8 +427,8 @@ void wxPseudoDC::DrawIdToDC(int id, wxDC *dc)
 // ----------------------------------------------------------------------------
 void wxPseudoDC::SetIdGreyedOut(int id, bool greyout)
 {
-    pdcObjectList::Node *pt = FindObjNode(id);
-    if (pt) pt->GetData()->SetGreyedOut(greyout);
+    pdcObject *obj = FindObject(id);
+    if (obj) obj->SetGreyedOut(greyout);
 }
 
 // ----------------------------------------------------------------------------
@@ -441,8 +436,8 @@ void wxPseudoDC::SetIdGreyedOut(int id, bool greyout)
 // ----------------------------------------------------------------------------
 bool wxPseudoDC::GetIdGreyedOut(int id)
 {
-    pdcObjectList::Node *pt = FindObjNode(id);
-    if (pt) return pt->GetData()->GetGreyedOut();
+    pdcObject *obj = FindObject(id);
+    if (obj) return obj->GetGreyedOut();
 	else return false;
 }
 
@@ -453,7 +448,7 @@ bool wxPseudoDC::GetIdGreyedOut(int id)
 PyObject *wxPseudoDC::FindObjectsByBBox(wxCoord x, wxCoord y)
 {
     //wxPyBlock_t blocked = wxPyBeginBlockThreads();
-    pdcObjectList::Node *pt = m_objectlist.GetFirst();
+    pdcObjectList::compatibility_iterator pt = m_objectlist.GetFirst();
     pdcObject *obj;
     PyObject* pyList = NULL;
     pyList = PyList_New(0);
@@ -481,7 +476,7 @@ PyObject *wxPseudoDC::FindObjects(wxCoord x, wxCoord y,
                                   wxCoord radius, const wxColor& bg)
 {
     //wxPyBlock_t blocked = wxPyBeginBlockThreads();
-    pdcObjectList::Node *pt = m_objectlist.GetFirst();
+    pdcObjectList::compatibility_iterator pt = m_objectlist.GetFirst();
     pdcObject *obj;
     PyObject* pyList = NULL;
     pyList = PyList_New(0);
@@ -599,7 +594,7 @@ PyObject *wxPseudoDC::FindObjects(wxCoord x, wxCoord y,
 // ----------------------------------------------------------------------------
 void wxPseudoDC::DrawToDCClipped(wxDC *dc, const wxRect& rect)
 {
-    pdcObjectList::Node *pt = m_objectlist.GetFirst();
+    pdcObjectList::compatibility_iterator pt = m_objectlist.GetFirst();
     pdcObject *obj;
     while (pt) 
     {
@@ -611,7 +606,7 @@ void wxPseudoDC::DrawToDCClipped(wxDC *dc, const wxRect& rect)
 }
 void wxPseudoDC::DrawToDCClippedRgn(wxDC *dc, const wxRegion& region)
 {
-    pdcObjectList::Node *pt = m_objectlist.GetFirst();
+    pdcObjectList::compatibility_iterator pt = m_objectlist.GetFirst();
     pdcObject *obj;
     while (pt) 
     {
@@ -628,7 +623,7 @@ void wxPseudoDC::DrawToDCClippedRgn(wxDC *dc, const wxRegion& region)
 // ----------------------------------------------------------------------------
 void wxPseudoDC::DrawToDC(wxDC *dc)
 {
-    pdcObjectList::Node *pt = m_objectlist.GetFirst();
+    pdcObjectList::compatibility_iterator pt = m_objectlist.GetFirst();
     while (pt) 
     {
         pt->GetData()->DrawToDC(dc);
